#include <stdio.h>
#include <getopt.h>
#include "flashlogo.h"

#define DEBUG

int main(int argc, char **argv)
{
	FILE *inputfile;
	int fsize = flashsize();
	struct logoblock2 logo;
	long size;
	int result;
	int c;
	char* buffer;
	
	logo.magic = LB_MAGIC;
	logo.flags = LB_SHOWGIF | LB_SHOWNW;
	logo.x = logo.y = 0;

#ifdef DEBUG
	if (sizeof(struct logoblock) != 65536)
	{
		fprintf(stderr, "%s: bad logo structure size... recompile? (%i)\n", argv[0], sizeof(struct logoblock));
		exit(1);
	}
#endif

	while (1)
	{
		static struct option long_options[] =
		{
			{ "no-netwinder", 0, 0, 'w' },
			{ "no-usergif", 0, 0, 'u' },
			{ "no-sound", 0, 0, 's' },
			{ "help", 0, 0, 'h' },
			{ "version", 0, 0, 'v' },
			{ 0, 0, 0, 0 }
		};
		
		c = getopt_long(argc, argv, "x:y:", long_options, NULL);
		
		if (c == -1)
			break;
			
		switch (c)
		{
		case 'x':
			logo.x = atoi(optarg);
			break;
		case 'y':
			logo.y = atoi(optarg);
			break;
		case 'u':
			logo.flags &= ~LB_SHOWGIF;
			break;
		case 'w':
			logo.flags &= ~LB_SHOWNW;
			break;
		case 's':
			logo.flags |= LB_NOSOUND;
			break;
		case 'h':
			printf(	"Usage: %s [OPTIONS]... [FILE]\n"
				"Write the specified GIF file to flash memory and configure startup graphic\n"
				"behaviour.\n"
				"\n"
				"      --no-netwinder          do not show NetWinder logo on startup.\n"
				"      --no-usergif            do not show user GIF file on startup. If this\n"
				"                               option is given, FILE is ignored.\n"
				"      --no-sound              do not play startup tune.\n"
				"  -x [X-COORD]                set x coordinate for top-left corner of user\n"
				"                               GIF file instead of 0.\n"
				"  -y [Y-COORD]                set y coordinate for top-left corner of user\n"
				"                               GIF file instead of 0.\n"
				"      --help                  display this help and exit.\n"
				"      --version               output version information and exit.\n"
				"\n", argv[0]);
			exit(1);
		case 'v':
			printf("logowrite2 1.10\n");
			exit(1);
		case '?':
			fprintf(stderr, "Try `%s --help' for more information.\n", argv[0], optarg, argv[0]);
			exit(1);
		}
	}
		
	if (logo.flags & LB_SHOWGIF)
	{
		if (optind >= argc)
		{
			fprintf(stderr, "%s: missing file argument\nTry `%s --help' for more information.\n", argv[0], argv[0]);
			exit(1);
		}
	
		inputfile = fopen(argv[optind], "rb");

		if (!inputfile)
		{
			fprintf(stderr, "%s: %s: Error opening input file.\n", argv[0], argv[optind]);
			exit(1);
		}
	
		printf("Checking GIF size... ");
	
		fseek(inputfile, 0, SEEK_END);
		size = ftell(inputfile);
		fseek(inputfile, 0, SEEK_SET);
		
		printf("%li bytes.\n", size);
	
		if (size > sizeof(logo.data2))
		{
			printf("\n");
			fprintf(stderr, "%s: GIF file is too large; maximum size is %u.\n", argv[0], sizeof(logo.data2));
			exit(1);
		}
			
		printf("Loading GIF into buffer... ");
		fflush(stdout);

		logo.size = size;
		fread(&(logo.data2), logo.size, 1, inputfile);
		
		printf("Done.\n");

		fclose(inputfile);
	}
	
	if (fsize < 0)
	{
		fprintf(stderr, "%s: unable to access flash device.\n", argv[0]);
		exit (1);
	}
	
	printf("Flash size: %i bytes.\n", fsize);
	
//	printf("Writing into flash at offset 0x%X... ", fsize-128*1024);
	printf("Writing into flash at offset 0x%X... ", fsize-62*1024);
	fflush(stdout);
	

	buffer = (char*)malloc (64*1024);
	if (buffer)
	{
	    //read in the 2K flash prefs strings
	//    memclr
	    result = flashread(buffer, fsize-64*1024, 2048);
	    memcpy (buffer+2048, (char*)&logo, 62*1024);
	    result = flashwrite(buffer, fsize-64*1024, 65536);

	}
	else
	    result = -1;
//	result = flashwrite((char *)&logo, fsize-128*1024, 65536);
	
	if (result < 0)
		printf("failed. (code %i)\n", result);
	else
		printf("%u bytes written.\n", result);
	

	if (buffer)
	    free(buffer);

	printf("Done.\n");
	
	return 0;
}
	